##	Factiva.filter.Aug2012.py
##
##  This formats the lead sentences of Reuters stories downloaded from Factiva in the 
##  format discussed below. Only stories with a "(Reuters) - " dateline and which are 
##  >64 and < 512 characters in length are included.
##
##  FACTIVA DOWNLOAD PARAMETERS
##
##  As we are still awaiting the return of the day when the bulk data providers allow 
##  files to be easily downloaded (note that when this was the case, we did not see
##  the collapse of civilization, nor of data providers, but this does not seem to have
##  figured in their calculations), stories need to be downloaded 100 at a time. Select 
##  "Reuters News" as the source, and use the "Subject" and "More Options" options to
##  eliminate categories you know you won't be using. You can select 100 stories at
##  a time -- use the check box in the upper left corner -- then use the following
##  options for the download
##	
##      Display options: "Full Article/Report plus indexing". This gives a display
##         which has "HD" in front of the headline and "PD" in front of the date.
##
##      Click the "Format for Saving" disk icon and use "Article format"
##
##      Decipher the increasingly difficult Capcha to convince the system that you are a 
##         non-robot who is exceedingly familiar with the Latin alphabet
##
##      Text will open in a new window, which can be saved. In FireFox, this is done 
##           using the "Save Page As..." with the "Web page, HTML only" option selected;
##           there are presumably similar options in other browsers
##
##  [These worked both at Maryland and in independent experimentation at Penn State]
##
##  If this doesn't work -- or when Factiva changes the format, which will happen sooner
##  or later, hence the name of the program -- you will need to modify the program. The  
##  present format is particularly easy to parse (thank you,Factiva...) and this program 
##  is considerably shorter than some of our earlier versions, and you may find some of  
##  those useful as examples. But first make sure you can't get these instructions to work.
##
##	TO RUN PROGRAM:
##
##	python Factiva.filter.Aug2012.py <command line options>
##
##	COMMAND LINE OPTIONS:
##
##  -i <filename>: process only <filename> rather than the complete list
##
##  -l <filename>: get the list of files from <filename> rather than "files.list.txt"
##
##  -o <filename>: output file name; default is "filter.outfile.txt"
##
## 	-d: Produces unformatted draft output for comparison with original file rather than
##      TABARI-formatted output
##
##  Example 1
##    python Factiva.filter.Aug2012.py
##    (reads the list of input files from "files.list.txt", the default file list, and 
##	   writes the draft output to "filter.outfile.txt", the default output file name)
##
##  Example 2
##    python Factiva.filter.Aug2012.py -l levant.list.txt -o levant.formatted.txt
##    (reads the list of input files from "levant.list.txt" and writes the output to
##	   "levant.formatted.txt")
##
##  Example 3
##    python Factiva.filter.Aug2012.py -i factiva.test.html -d
##    (reads the single input file from "factiva.test.html" and writes the draft output to
##	   "filter.outfile.txt", the default output file name)
##
##	INPUT FILES
##
##  files.list.txt: list of the files to process. The last line in the file should be
##                  blank (that is, only a return \n)
##  
##	OUTPUT FILES
##
##  TABARI-formatted records (default) or draft output with the -d command.   
##
##	PROGRAMMING NOTES: None
##
##	SYSTEM REQUIREMENTS
##	This program has been successfully run under Mac OS 10.6; it is standard Python 2.5
##	so it should also run in Unix or Windows. 
##
##	PROVENANCE:
##	Programmer: Philip A. Schrodt
##				Dept of Political Science
##				Pennsylvania State University
##				227 Pond Laboratory
##				University Park, PA, 16802 U.S.A.
##				http://eventdata.parusanalytics.com
##
##	Copyright (c) 2012	Philip A. Schrodt.	All rights reserved.
##
## This project was funded in part by National Science Foundation grant 
## SES-1004414
##
##	Redistribution and use in source and binary forms, with or without modification,
##	are permitted under the terms of the GNU General Public License:
##	http://www.opensource.org/licenses/gpl-license.html
##
##	Report bugs to: schrodt.parusanalytics.com
##
##	REVISION HISTORY:
##	2-August-12:	Initial version
##
##	----------------------------------------------------------------------------------

import sys
import textwrap
import calendar

# ======== global initializations ========= #

# search patterns
partstart  = "<div class=\"article enArticle\">"  # start of article
pheadline  = "<b class=\'enHeadline\'>"           # headline
pdatefield = "<b>PD</b>&nbsp;</td><td>"           # date
pdocfield  = "Document LBA00"                     # Document ID; also signals end of article
pparagraph = "<p class=\"articleParagraph enarticleParagraph\">" # lead sentence
pReuter = "(Reuters) - "                          # Reuters dateline

dirfile = "files.list.txt"   	# default file list
outfile = "filter.outfile.txt"  # default output
dosingle = False
doformated  = True

wrapper = textwrap.TextWrapper(width = 79)
months = calendar.month_name

# ============ function definitions ================ #

# source for following code to strip out HTML tags
# http://stackoverflow.com/questions/753052/strip-html-from-strings-in-python
# Accessed: 3-Aug-2012

from HTMLParser import HTMLParser 

class MLStripper(HTMLParser):
    def __init__(self):
        self.reset()
        self.fed = []
    def handle_data(self, d):
        self.fed.append(d)
    def get_data(self):
        return ''.join(self.fed)

def strip_tags(html):
    s = MLStripper()
    s.feed(html)
    return s.get_data()

# ============ main program =============== #

if len(sys.argv) > 1:   # process command line options
	if ("-l" in sys.argv):
		dirfile = sys.argv[sys.argv.index("-l")+1]	
		print "-l: Reading file list from",dirfile
	if ("-i" in sys.argv):
		dosingle = True
		infile = sys.argv[sys.argv.index("-i")+1]+"+" # this gets removed	
		print "-i: Processing single file",infile[:-1]
	if ("-o" in sys.argv):
		outfile = sys.argv[sys.argv.index("-o")+1]	
		print "-o: Output file",outfile
	if ("-d" in sys.argv):
		doformated = False	
		print "-d: Producing unformatted draft output"

fout = open(outfile,'w')

if not dosingle: 
	try: 
		fdir = open(dirfile,'r')
	except IOError:
		print "\aError: Could not find the file list file",dirfile
		sys.exit()	
	infile = fdir.readline() 

while len(infile) > 1:  # loop through the file list

	try: 
		fin = open(infile[:-1],'r')
	except IOError:
		print "\aError: Could not find the file",infile[:-1]
		sys.exit()	
	print "\nReading file:",infile,
	if not doformated:
		fout.write("\n---------------------------------------\n")
		fout.write("    INPUT FILE: "+infile)
		fout.write("---------------------------------------\n")

	caseno = 1
	line = fin.readline()
	endidx = 0 
	noarticle = False
	while len(line) > 0:  # loop through the file
		idx = line.find(partstart, endidx)  # check for start of article
		if idx >=0:
#			print "Mk1",idx  # debug
			newart = True
			noarticle = False
			story = ""   # holds the story contents
			caseno += 1
#			if caseno > 16: sys.exit()  # debug
			docidx = line.find(pdocfield,idx)  # check for start of article
			while (docidx < 0) and (len(line) > 0): # read multiple lines if needed
				if newart:
					story += line[idx:]
					idx = 0
					newart = False
				else: story += line
				line = fin.readline() 
				docidx = line.find(pdocfield)
			if docidx < 0: 
				print "Error: could not find document label" # eof, but shouldn't hit this
				break 
	#		print "Mk3",docidx
			endidx = line.find("</td>",docidx)  # this is the last part of the article we need
			story += line[idx:endidx]

            ### process story fields ###
	
			pdx = story.find(pheadline)
			shead = story[pdx+len(pheadline):story.find("</b>",pdx+1)]
			print "HD:",shead, 

			pdx = story.find(pdocfield)
			sdoc = story[pdx+9:]

			pdx = story.find(pparagraph)
			sent = story[pdx+len(pparagraph):story.find("</p>",pdx+1)]

			pdx = story.find(pdatefield)
			thedate = story[pdx+len(pdatefield):story.find("</td>",pdx+len(pdatefield)+1)]
			parts = thedate.split()
			if (len(parts[0]) == 1): sday = "0"+parts[0]
			else: sday = parts[0]
			for ka in range(13):    # why didn't months.index() work here?
				if months[ka] == parts[1]:
					mon = ka
					break
			if mon < 10: smon = "0"+ str(mon)
			else: smon = str(mon)
			sfdate = parts[2]+smon+sday

			if sent.find(pReuter) >= 0:
				sent = sent[sent.find(pReuter)+len(pReuter):]
				sent2 = strip_tags(sent)
				print " OK"				
			else: 
				sent2 = "--"
				print " DISCARD"

			if doformated:
				if len(sent2) > 64 and len(sent2) < 512 : # this handles both the discard and the odd "sentence" which is too short or too long
					fout.write(sfdate+" "+sdoc+"\n")
					linelist = wrapper.wrap(sent2)
					for ka in range(len(linelist)): fout.write(linelist[ka]+" \n")
					fout.write("\n")			
			else:
				fout.write("\n+++++++++++++++++++\n\n")
				fout.write("HEADLINE: "+shead+"\n")
	
				fout.write("DATE:     "+thedate+"\n")
				fout.write("DATE FMTD: "+sfdate+"\n")				
	
				fout.write("DOC ID:   "+sdoc+"\n")
	
				fout.write("LEAD:     "+sent+"\n")
				if len(sent2) > 64 and len(sent2) < 512 :
					fout.write("OUTPUT:\n")
					linelist = wrapper.wrap(sent2)
					for ka in range(len(linelist)): fout.write(linelist[ka]+" \n")			
				else: fout.write("OUTPUT:   -- DISCARD --\n")

		else: 
			line = fin.readline()  
			endidx = 0
	
	fin.close()
	if noarticle :
		print "\aError: could not find any articles in the file",infile,
		print "Please check the file formating and re-run"
		print "Ending processing at this point"
		break 
	if dosingle: break  # normal termination when doing only one file 
	else: infile = fdir.readline() 

fout.close()
print "Finished"
